<?php

namespace Drupal\instagram_field\Plugin\Field\FieldFormatter;

use Drupal\Core\Field\FormatterBase;
use Drupal\Core\Field\FieldItemListInterface;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Url;
use Drupal\Component\Serialization\Json;

/**
* Plugin implementation of the 'instagramfield_link' formatter.
*
* @FieldFormatter(
*   id = "instagramfield_formatter",
*   label = @Translation("Instagram recent"),
*   field_types = {
*     "instagramfield"
*   }
* )
*/
class InstagramFieldFormatter extends FormatterBase {

    private $thumbsDirectory = 'public://instagram_thumbnails';


    /**
     * Get recent posts from instagram
     */
    public function viewElements(FieldItemListInterface $items, $langcode) {
        $config = \Drupal::config('config.instagram_field');
        $client = \Drupal::httpClient();
        $request = $client->request('GET',
            'https://api.instagram.com/v1/users/self/media/recent/', [
                'query' => [
                    'access_token' => $config->get('accesstoken'),
                    'count' => 1,
                ],
            ]
        );
        $result = json::decode($request->getBody());
        foreach ($result['data'] as $key => $value) {
            if (isset($items[$key])){
                $this->load_image($value['id'], $value['images'][$config->
                    get('imageresolution')]['url']);
                $items[$key]->setValue(['instagramid'=>$value['id'],
                    'instagramlink'=>$value['link']], TRUE);
                $items[$key]->getEntity()->save();
            }
        }


        $element = [];
        foreach ($items as $delta => $item) {
            $element[$delta] = [
                '#type' => 'html_tag',
                '#tag' => 'A',
                '#attributes' => [
                    'href' => $items[$delta]->getValue()['instagramlink'],
                ],
                'content' => [
                    'img' => [
                        '#type' => 'html_tag',
                        '#tag' => 'img',
                        '#attributes' => [
                            'src' => file_create_url($this->thumbsDirectory .
                                '/' . $items[$delta]->getValue()['instagramid']
                                . '.jpg'),
                        ],
                    ],
                ],
                '#cache' => [
                    'max-age' => $config->get('cachetime') * 60
                ],
            ];
        }
        return $element;
    }

    /**
     * Load image from disk or download it from instagram
     */
    private function load_image($imageid, $url){
        $local_uri = $this->thumbsDirectory . '/' . $imageid . '.jpg';
        if (!file_exists($local_uri)) {
            file_prepare_directory($this->thumbsDirectory, FILE_CREATE_DIRECTORY);
            try {
                $thumbnail = \Drupal::httpClient()->request('GET', $url);
                file_unmanaged_save_data((string) $thumbnail->getBody(),
                    $local_uri);
            }
            catch (\Exception $e) {

            }
        }
    }
}
